%Constrained optimization using Newton's method
%
%function [xs,lams,valid] = amsnewton_constrainedopt(f_obj, f_const, x0,
%fpars, N, M, solpars);
%
%Inputs:
%f_obj - function pointer to scalar return value objective function of the
%form s = f_obj(x,fpars)
%f_const - function pointer to M-vector valued constraint function of the
%form colvect = f_const(x,fpars)
%x0 - initial search state (N-column-vector)
%N - size of search space
%M - size of constraints (NDOF = N-M)
%solpars - a vector of solver control parameters. Defaults can be generated
%by solpars = amsnewton_constrainedopt_defpars();
%
%Outputs:
%xs - solution x
%lams - solution lagrange multipliers for 
%       L(x,lam) = f_obj(x) - lam'*f_const(x)
%valid - a flag indicating validity of solution: 1 is solution is
%converged.
function [xs,lams,valid] = amsnewton_constrainedopt(f_obj, f_const, x0, fpars, N, M, solpars);

valid = 0; %Something is wrong if this gets returned
xs = x0;
lams = zeros(1,M);

solpars = int_setdef(solpars);
%N - nubmer of variables, also size of f_obj
%M - number of constraints, also size of f_const, lam
dx = solpars.dx;

if(size(x0,1)~=N)
    fprintf('Error! Size of X =/= N, sz(X) = %d\n',size(x0,2));
    return;
end

xp = x0;
lamp = zeros(M,1);
%xold = xp;
rel = 1E20;
Lold = 1E20;
qf = 0;
I = 0;
pold = [0,0,0]';

while(qf == 0)
    xold = xp;
    p = zeros(M+N,1); %search direction vector
    L0 = int_lagrangian(f_obj,f_const,xp,lamp,fpars);
    
    %Calculate Hessian - I can hear my processor crying
    hgab = int_hessian(f_obj,f_const,xp,lamp,dx,N,M,fpars);
    H = hgab.H;
    G = hgab.G;
    A = hgab.A;
    B = hgab.B;
    ZZ = zeros(M,M);
    
    H_L = [H, G'; G, ZZ];
    v_L = [-A;-B];
    

    
    %Get search direction
    if(abs(det(H_L))<solpars.dettol||abs(rcond(H_L))<solpars.condtol)
        if(solpars.verbose>1)
            fprintf('Warning! Determinant of H_L is near zero!\n');
            fprintf('In this case, we move in gradient direction.\n');
        end

        if(norm([A;B])>solpars.vtol)
            %We have a gradient to the Lagrangian, but a singular
            %hessian
            pp = [A;B]; %pp is gradient of the lagrangian.
            %%question - is there curveature along the gradient descent
            %%vector direction?
            zp = [xp;lamp];
            zp_1 = zp + pp./norm(pp).*dx;
            zp_2 = zp + pp./norm(pp).*2.*dx;
            xp_1 = zp_1(1:N);
            xp_2 = zp_2(1:N);
            lamp_1 = zp_1(N+1:N+M);
            lamp_2 = zp_2(N+1:N+M);
            LP1 = int_lagrangian(f_obj,f_const,xp_1,lamp_1,fpars);
            LP2 = int_lagrangian(f_obj,f_const,xp_2,lamp_2,fpars);
            ppc = (L0 + LP2 - 2*LP1)/(4*dx*dx); %Curveature
            if(abs(ppc)>solpars.ctol)
                %We have curveature, we can proceed according to a 1d
                %Newton's method
                if(solpars.verbose>1)
                    fprintf('Case: We have curvature. |pp| = %1.6g, ppc = %1.6g\n',norm(pp),ppc);
                end
                p = -pp/ppc;
            else
                %We do not have curveature of lagrangian in search
                %direction.
                %Do we ascend or descend?
                %For this, we need to know if we were ascending or descending
                %last iteration
                %We do not have curveature. We should move randomly?
                if(solpars.verbose>1)
                    fprintf('Case: We do not have curvature. |pp| = %1.6g, ppc = %1.6g\n',norm(pp),ppc);
                end
                p = 2.0*rand(N+M,1)-ones(N+M,1); %pick a random direction
                p = p./norm(p);  
            end
        else
            %If there is no gradient in the Lagrangian, then technically,
            %we have met our end condition.
            if(solpars.verbose>1)
               fprintf('Case: We do not have a gradient. |pp| = %1.6g\n',norm(pp));
            end
            qf = 1;
            xs = xp;
            lams = lamp;
            valid = 1;
        end
    else
        %We have nonsingular H_L, and a gradient. We're good.
        p = (H_L^-1)*v_L;
    end

    %Decide if we should force a descent instead of ascent?
    if(solpars.fdesc==1)
        %Check that p is a descent direction:
        %For some reason, my lagrangian gradient is buggy
        %if((-v_L'*p)>0)
        %    if(solpars.verbose>1)
        %        fprintf('Reversing p\n');
        %    end
        %    
        %    %p turns out to be an ascent direction, not a descent direction
        %    p = -p; %search the other way
        %end
        
        %Despite my best efforts, I cannot ensure p is a descent direction
        %that way. I'll try verifying directly
        znp = [xp;lamp] + solpars.alpha.*p.*dx;
        znm = [xp;lamp] - solpars.alpha.*p.*dx;
        Lp = int_lagrangian(f_obj,f_const,znp(1:N),znp(N+1:N+M),fpars);
        Lm = int_lagrangian(f_obj,f_const,znm(1:N),znm(N+1:N+M),fpars);
        if(Lp>Lm)
            p = -p;
            if(solpars.verbose>1)
                fprintf('Reversing p\n');
            end
        end
    end
    
    %Begin linesearch
    xpp = xp + solpars.alpha.*p(1:N);
    lampp = lamp + solpars.alpha.*p(N+1:N+M);
    L1 = int_lagrangian(f_obj,f_const,xpp,lampp,fpars);
    
    %If we are forcing descent, then perform a linesearch along descent
    %direction to ensure we have L less than we started with
    if(solpars.fdesc==1)
        II = 0;
        while(L1>L0&&II<solpars.maxinneriter)
            p = p.*solpars.descrelax;
            xpp = xp + solpars.alpha.*p(1:N);
            lampp = lamp + solpars.alpha.*p(N+1:N+M);
            L1 = int_lagrangian(f_obj,f_const,xpp,lampp,fpars);
            II = II + 1;
            if(solpars.verbose>1)
                fprintf('\tInner %d, |p|=%1.6g L = %1.6g, L0 = %1.6g\n',II,norm(p),L1,L0);
            end
        end
    end
    %%End linesearch in case of fdesc=1
    
    if(solpars.verbose>0)
        fprintf('I=%d, L=%1.6g\n',I,L0);
    end
    
    %Figure out if we can terminate now
    rel = abs(L0-Lold); %should be positive, but check to make sure
    if(solpars.fdesc==1)
        b1 = rel<solpars.tol&&(L0<Lold);
    else
        b1 = rel<solpars.tol;
    end
    
    if(b1)
        qf = 1; %success!
        xs = xpp;
        lams = lampp;
        valid = 1;
    end
    
    I = I + 1;
    xp = xpp;
    lamp = lampp;
    Lold = L0;
    pold = p;
    
    %Termination Conditions
    if(I>solpars.maxiter)
        qf = -1; %Terminating without convergence
        xs = xp;
        lams = lamp;
        valid = -1; %No convergence
    end
end


return

function L = int_lagrangian(f_obj,f_const,xp,lamp,fpars)
    L = f_obj(xp,fpars) + lamp'*f_const(xp,fpars);
return

%Computes the lagrangian hessian and the gradients of the objective
%function
%Used to construct the constrained lagrangian hessian matrix
%
%Depending on the sort of problem, computing the Hessian of f_obj could be
%expensive!
function out = int_hessian(f_obj, f_const, xp, lamp, dx, N, M, fpars)
%Hji = d^2L/dx_j*dxi
%Gji = d^2L/dx_i*dlam_j
%Aj = dL/dxj = 
%Bj = dL/dlamj = f_const_j

%Calling F is expensive, so we'll keep all variations here so we only have
%to grab them once!
out = struct();

f0 = f_obj(xp,fpars);   %scalar
c0 = f_const(xp,fpars); %1 x M vector

fp = {};
fm = {};
cp = {};
cm = {};
fdd1 = {};
fdd2 = {};
fdd3 = {};
fdd4 = {};
cdd1 = {};
cdd2 = {};
cdd3 = {};
cdd4 = {};

%Variations along each axis
for I = 1:N
    xp_p = int_xvar_i(xp,dx,I);
    xp_m = int_xvar_i(xp,-dx,I);
    
    fp{I} = f_obj(xp_p,fpars);
    fm{I} = f_obj(xp_m,fpars);
    cp{I} = f_const(xp_p,fpars);
    cm{I} = f_const(xp_m,fpars);
end

%Second order variations (N*(N-1)) of them!
%to calculate the cross-terms in the hessian
for I = 1:N
    for J = 1:N
        if(J<I)
            x1 = int_xvar_ij(xp,dx,dx,I,J);
            x2 = int_xvar_ij(xp,-dx,dx,I,J);
            x3 = int_xvar_ij(xp,-dx,-dx,I,J);
            x4 = int_xvar_ij(xp,dx,-dx,I,J);
            
            fdd1{I}{J} = f_obj(x1,fpars);
            fdd2{I}{J} = f_obj(x2,fpars);
            fdd3{I}{J} = f_obj(x3,fpars);
            fdd4{I}{J} = f_obj(x4,fpars);
            cdd1{I}{J} = f_const(x1,fpars);
            cdd2{I}{J} = f_const(x2,fpars);
            cdd3{I}{J} = f_const(x3,fpars);
            cdd4{I}{J} = f_const(x4,fpars);
        else
            fdd1{I}{J} = [];
            fdd2{I}{J} = [];
            fdd3{I}{J} = [];
            fdd4{I}{J} = [];
            cdd1{I}{J} = [];
            cdd2{I}{J} = [];
            cdd3{I}{J} = [];
            cdd4{I}{J} = [];
        end 
    end
end

%Calculate 1st derivatives
dFdxi = zeros(N,1);
dckdxi = zeros(M,N);
for I = 1:N
    dFdxi(I) = (fp{I}-fm{I})./2./dx;
    cc = (cp{I}-cm{I})./2./dx;
    for K = 1:M
        dckdxi(K,I) = cc(K);
    end
end


%Calculate second derivatives of objective function
d2Fdxidxj = zeros(N,N);
for I = 1:N
    for J = 1:N
        if(I==J)
            d2Fdxidxj(I,J) = (fp{I} + fm{I} - 2.*f0)./dx.^2;
        elseif(J<I)
            d2Fdxidxj(I,J) = (fdd1{I}{J} + fdd3{I}{J} - fdd2{I}{J} - fdd4{I}{J})./4./dx.^2;
        elseif(J>I)
            d2Fdxidxj(I,J) = (fdd1{J}{I} + fdd3{J}{I} - fdd2{J}{I} - fdd4{J}{I})./4./dx.^2;
        end
    end
end

%Caluclate second derivatives of constraint function
d2ckdxidxj = zeros(M,N,N);
for I = 1:N
    for J = 1:N
        if(I==J)
            cc = (cp{I} + cm{I} - 2.*c0)./dx.^2;
        elseif(J<I)
            cc = (cdd1{I}{J} + cdd3{I}{J} - cdd2{I}{J} - cdd4{I}{J})./4./dx.^2;
        elseif(J>I)
            cc = (cdd1{J}{I} + cdd3{J}{I} - cdd2{J}{I} - cdd4{J}{I})./4./dx.^2;
        end
        for K = 1:M
            d2ckdxidxj(K,I,J) = cc(K);
        end
    end
end

%Now we have all our gradients and curvatures
%dFdxi
%dckdxi
%d2Fdxidxj
%d2ckdxidxj
H = zeros(N,N);
G = zeros(M,N);
A = zeros(N,1);
B = zeros(M,1);

for I = 1:N
    for J = 1:N
        S = 0;
        for K = 1:M
           S = S-lamp(K)*d2ckdxidxj(K,I,J);
        end
        H(I,J) = d2Fdxidxj(I,J)+S;
    end
end

G = -dckdxi;

for I = 1:N
    S = 0;
    for K = 1:M
        S = S-lamp(K)*dckdxi(K,I);
    end
    A(I) = dFdxi(I)+S;
end

B = -c0;

out.H = H;
out.G = G;
out.A = A;
out.B = B;
        
return

function xvar = int_xvar_i(xc,dx,I)
xvar = xc;
xvar(I) = xc(I) + dx;
return

function xvar = int_xvar_ij(xc,dxi,dxj,I,J)
xvar = xc;
xvar(I) = xvar(I) + dxi;
xvar(J) = xvar(J) + dxj;
return

function solpars = int_setdef(solpars)
if(nargin<1)
    solpars = struct();
end

if(~isfield(solpars,'dx'))
    solpars.dx = 1E-6; %delta-x over which to vary input parameters
end

if(~isfield(solpars,'tol'))
    solpars.tol = 1E-3; %tolerance for |f(x_i)-f(x_i-1)| to return converged solution
end

if(~isfield(solpars,'maxiter'))
    solpars.maxiter = 10000; %max # of iterations - will give up after these
end

if(~isfield(solpars,'verbose'))
    solpars.verbose = 0; %don't talk (1 - some, 2- extensive (debug))
end

if(~isfield(solpars,'fdesc'))
    solpars.fdesc = 0; %Force descent on each iteration by reversing direction if necessary.
    %If you don't do this, the solver will find an extremum, not a minimum
end

if(~isfield(solpars,'descrelax'))
    solpars.descrelax = 0.5; %if travelling along p doesn't give you a value
    %less than the one you started with, back off by this factor
end

if(~isfield(solpars,'alpha'))
    solpars.alpha = 0.5; %underrelaxation factor
end

if(~isfield(solpars,'dettol'))
    solpars.dettol = 1E-4; %Determinant of lagrangian hessian matrix
end

if(~isfield(solpars,'condtol'))
    solpars.condtol = 1E-15; %Determinant of lagrangian hessian matrix
end

if(~isfield(solpars,'maxinneriter'))
    solpars.maxinneriter = 50;
end

if(~isfield(solpars,'vtol'))
    solpars.vtol = 1E-7;
end

if(~isfield(solpars,'ctol'))
    solpars.ctol = 1E-7;
end

return