%Broyden Fletcher Goldfarb Shannon Algorithm
%Uses gradients of the test function at two points to come up with an
%approximation of the Hessian matrix. This is updated at each iteration
%using the previous Hessian approximation.
%
%hess - a struct containing the old hessian, inverse hessian, gradient, and
%x coordinate. If hess is blank, the algorithm assumes it is the initial
%step and will initialize hess. pk will be taken to be the gradient
%direction.
%   hess.x_km1 - x_{k-1}
%   hess.H_km1 - H_{k-1}
%   hess.Hi_km1 - H^-1_{k-1}
%   hess.p_km1 - grad(F)_{k-1}
%
%valid - which type of descent direction are we using?
%   valid = 2:      Gradient is zero - you're done!
%   valid = 1:      pk determined by normal newton step
%   valid = 0:      pk determined by gradient and curvature
%   valid = -1:     pk is -gradient, but no curvature was seen
%
%This is a blackbox algorithm for me, and it grates that I don't see where
%this comes from or how it works internally!
function [pk, hessk, valid] = amsopt_bfgh_dirupdate(fpnt,x,dx,fopt,N,hess,solpars)
pk = zeros(N,1);
valid = -2;
hessk = struct();

solpars = int_solpars_def(solpars); %get defaults

[grad,crv] = int_gradient(fpnt,x,dx,fopt,N); %get current gradient and curvature

if(norm(grad)<solpars.vtol)
    %We are already converged. The gradient is less than the vector
    %tolerance.
    pk = zeros(N,1);
    hess = struct();
    hessk = struct();
    valid = 2;
    return;
end

if(~isfield(hess,'x_km1'))
    hess = struct(); %this is the first iteration - intiialize hess
    hess.x_km1 = x;
    hess.H_km1 = eye(N,N);
    hess.Hi_km1 = eye(N,N);
    
    H = eye(N,N);
    Hi = eye(N,N);
    
    pk = -Hi*grad; %Normal newton step is default
    valid = 1;
    
    hess.p_km1 = pk;
    hess.grad_km1 = grad;
    
    hessk = hess;
else
    %This is a subsequent iteration: We must construct H, Hi
    s = x - hess.x_km1;
    y = grad - hess.grad_km1;
    
    %New hessian update
    M1 = (y*y')/(y'*s);
    M2 = hess.H_km1*s*s'*hess.H_km1/(s'*hess.H_km1*s);
    Hk = hess.H_km1 + M1 + M2;
    
    %New inverse hessian update
    M3 = (eye(N,N)-s*y'/(y'*s));
    M4 = (eye(N,N)-y*s'/(y'*s));
    M5 = s*s'/(y'*s);
    Hik = M3*hess.Hi_km1*M4+M5;
    
    detbad = 0;
    if(abs(rcond(Hik))>solpars.condtol)
        if(abs(det(Hik))>solpars.dettol)
            %condition is good, normal newton step
            pk = -Hik*grad; %Normal newton step is default
            valid = 1;
        else
            %determinant is bad
            detbad = 1;
        end
    else
        %condition number is bad
        detbad = 1;
    end
    
    if(detbad == 1)
        %take a step along gradient direction
        if(abs(crv)>solpars.ctol)
            pk = -grad/crv;
            valid = 0;
        else
            %we don't even have curvature
            pk = -grad;
            valid = -1; %not really trustworthy
        end
    end
            
    %populate hessk
    hessk = struct();
    hessk.Hi_km1 = Hik;
    hessk.H_km1 = Hk;
    hessk.grad_km1 = grad;
    hessk.p_km1 = pk;
    hessk.x_km1 = x;
            
end



return

%Obtain current gradient (and curvature along gradient direction, while I'm
%at it
function [grad,crv] = int_gradient(fpnt,x,dx,fopt,N)

f0 = fpnt(x,fopt);
fp = zeros(N,1);
fm = zeros(N,1);

for I = 1:N
    [xp, xm] = int_xvariant_a(x,dx,I);
    fp(I) = fpnt(xp,fopt);
    fm(I) = fpnt(xm,fopt);
end

grad = zeros(N,1);

for I = 1:N
    grad(I) = (fp(I) - fm(I))/2/dx;
end

xp1 = x-grad./norm(grad).*dx;
xp2 = x-2.0.*grad./norm(grad).*dx;
fp1 = fpnt(xp1,fopt);
fp2 = fpnt(xp2,fopt);
crv = (fp2+f0-2*fp1)/dx/dx;

return

function [xp,xm] = int_xvariant_a(x,dx,I)
    xp = x;
    xm = x;
    if(I>=1&&I<=size(x,1))
        xp(I) = xp(I) + dx;
        xm(I) = xm(I) - dx;
    end
return

%Set up solver parameter defaults if they aren't set up already
function solpars = int_solpars_def(solpars)

if(~isfield(solpars,'verbose'))
    solpars.verbose = 0; %Degree of detail in comments
end
if(~isfield(solpars,'condtol'))
    solpars.condtol = 1E-15; %min condition (rcond) of Hessian matrix
end
if(~isfield(solpars,'dettol'))
    solpars.dettol = 1E-5; %minimum determinant of H
end
if(~isfield(solpars,'ctol'))
    solpars.ctol = 1E-7; %minimum curvature in line-search case to be considered curved
end
if(~isfield(solpars,'vtol'))
    solpars.vtol = 1E-7; %minimum magnitude of gradient
    %if it is less than this, signal the method to stop
    %by setting valid = 2
end

return