%Linesearch along a search direction, using the first wolfe condition and
%Armijo backtracking to determine sufficient decrease/increase.
%
%Unless solpars.fdesc = 1 is set, the linesearch is ambivalent between
%ascending and descending. 
%
%Implements 1st wolfe condition: I understand that one.
function [x2, a, flg] = amsopt_lnsrch_wolfe(fpnt,x,dx,fopt,p,solpars)

solpars = int_defaults(solpars);
verbose = solpars.verbose;

%Initially, p is taken to be the scaled newton-raphson search direction.
%The gradient along the direction p is evaluated
phat = p./norm(p);
[grd0,crv0] = int_lngrd(fpnt,x,dx,0,p,phat,fopt);
ngrd0 = grd0*norm(p);
f0 = fpnt(x,fopt);

%Are we ascending or descending?
bdesc = 0;
if(grd0<0)
    bdesc = 1; %we are descending?
    
    if(verbose>3)
        fprintf('\tThis is a descent direction\n');
    end
else
    if(verbose>3)
        fprintf('\tThis is an ascent direction\n');
    end
end

if(solpars.fdesc == 1) %If we force descent at each iteration, 
    if(bdesc == 0)
        grd0 = -grd0;
        ngrd0 = -ngrd0;
        p = -p;
        phat = -phat;
        bdesc = 1;
        if(verbose>3)
            fprintf('\tReversing direction...\n');
        end
    end
end

%a = 1;
if(abs(crv0)>solpars.ctol)
    a = abs(grd0/crv0)/norm(p);
else
    a = 1;
end

I = 0;
b1 = 0; b2 = 0;
while((b1 == 0 || b2 == 0) && I<solpars.wlf_itermax)
    b1 = int_wolfec1_a(fpnt,x,fopt,a,p,f0,ngrd0,bdesc,solpars.wlf_c1);
    b2 = int_wolfec2_a(fpnt,x,dx,fopt,a,p,phat,ngrd0,solpars.wlf_c2);
    if(verbose>3)
        fprintf('\t%d: Wolfe conditions: [%d %d]\n',I,b1,b2);
    end
    if(~b1)
        %if 1st condition is not met, backtrack
        a = a*solpars.wlf_r1;
        if(verbose>3)
            fa = fpnt(x+a.*p,fopt);
            fprintf('\t Backtrack: a=%1.6g fa=%1.6g\n',a,fa);
        end
    else
        if(~b2)
            %if second condition is not met, advance
            a = a*solpars.wlf_r2;
            
            if(verbose>3)
                fa = fpnt(x+a.*p,fopt);
                fprintf('\t Advance: a=%1.6g fa=%1.6g\n',a,fa);
            end
        end
    end
    I=I+1;
end

%output solution
if(I>=solpars.wlf_itermax)
    flg = 0;
else
    flg = 1;
end
x2 = x + a.*p;


return

function b1 = int_wolfec1_a(fpnt,x,fopt,a,p,f0,ngrd0,bdesc,c1)
b1 = 0;

fa = fpnt(x+a.*p,fopt);
if(bdesc==1)
    %descending
    b1 = (fa <= f0 + c1*a*ngrd0);
else
    %ascending
    b1 = (fa >= f0 + c1*a*ngrd0);
end

return

function b1 = int_wolfec2_a(fpnt,x,dx,fopt,a,p,phat,ngrd0,c2)
b1 = 0;
ngrda = int_lngrd(fpnt,x,dx,a,p,phat,fopt)*norm(p);
b1 = abs(ngrda)>=abs(ngrd0)*c2;
return

function [grd,crv] = int_lngrd(fpnt,x,dx,a,p,phat,fopt)
f0 = fpnt(x+a.*p,fopt);
f1 = fpnt(x+a.*p+phat.*dx,fopt);
f2 = fpnt(x+a.*p+phat.*dx.*2,fopt);
grd = (f1-f0)/dx;
crv = (f2+f0-2*f1)/dx/dx;
return

function solpars = int_defaults(solpars)

if(~isfield(solpars,'wlf_c1'))
    solpars.wlf_c1 = 0.1; %1st wolfe condition parameter
end
if(~isfield(solpars,'wlf_c2'))
    solpars.wlf_c2 = 0.8; %2nd wolfe condition parameter
end
if(~isfield(solpars,'wlf_r1'))
    solpars.wlf_r1 = 0.50; %Armijo backtracking coefficient
end

if(~isfield(solpars,'wlf_r2'))
    solpars.wlf_r2 = 1.21; %Advance coefficeint
end

if(~isfield(solpars,'wlf_itermax'))
    solpars.wlf_itermax = 25; %Maximum wolfe iterations
end

if(~isfield(solpars,'fdesc'))
    solpars.fdesc = 0; %Force descent?
end

if(~isfield(solpars,'ctol'))
    solpars.ctol = 1E-7; %Tolerance for zero curvature
end

return
