%Newton's method direction update
%Returns the direction chosen by Newton's method (classic formulation with
%full hessian) for a scalar function fpnt(x,fpars) taking N parameters x(I)
%and returning a scalar value.
%
%Hess is a cell struct containing {H, A} where H is the hessian, and A is the gradient of fpnt
%
%function [p, valid, hess] = ams_newtondirupdate(fpnt,x,dx,N,fopt,solpars)
%Outputs
%p - scaled search direction
%valid: type of search direction
%   valid = 2:     There is no gradient: You are at a minimum!
%   valid = 1:     Standard newton's method step by inverting hessian
%   valid = 0:     Hessian was singular, but curvature existed along search
%   direction
%   valid = -1:    No curvature to go off of, returning p = -grad(F)
%hess.H = second derivative hessian of fpnt(x,fopt)
%hess.A = first derivative gradient of fpnt(x,fopt)
function [p, valid, hess] = amsopt_newtondirupdate(fpnt,x,dx,N,fopt,solpars)

if(nargin<6)
    solpars = struct();
end
if(~isfield(solpars,'verbose'))
    solpars.verbose = 0; %Degree of detail in comments
end
if(~isfield(solpars,'rtol'))
    solpars.condtol = 1E-15; %min condition (rcond) of Hessian matrix
end
if(~isfield(solpars,'dettol'))
    solpars.dettol = 1E-5; %minimum determinant of H
end
if(~isfield(solpars,'ctol'))
    solpars.ctol = 1E-7; %minimum curvature in line-search case to be considered curved
end
if(~isfield(solpars,'vtol'))
    solpars.vtol = 1E-7; %minimum magnitude of gradient
    %if it is less than this, signal the method to stop
    %by setting valid = 2
end

hess = int_hessian(fpnt,x,dx,N,fopt);

gsrch = 0;
verbose = solpars.verbose;

if(abs(det(hess.H))<solpars.dettol)
    if(verbose>2)
        fprintf('Warning: det(H)<%1.6g!\n',solpars.dettol);
    end
    gsrch = 1;
elseif(abs(rcond(hess.H))<solpars.condtol)
    if(verbose>2)
        fprintf('Warning: rcond(H)<%1.6g!\n',solpar.condtol);
    end
    gsrch = 1;
end

if(norm(hess.A)<solpars.vtol)
    p = zeros(N,1);
    valid = 2; %You're done!
else
    if(gsrch == 0)
        p = -(hess.H^-1)*hess.A;
        valid = 1;
    else
        %In this case, the Hessian is badly scaled or close to singular.
        %Rather than trying to invert it, look along the gradient direction,
        %and see if there is curvature. Go along gradient as if 1d newton's
        %method.
        xp = x + (hess.A)./norm(hess.A).*dx;
        xpp = x + 2.*(hess.A)/norm(hess.A).*dx;
        f0 = fpnt(x,fopt);
        fp = fpnt(xp,fopt);
        fpp = fpnt(xpp,fopt);
        crv = (fpp + f0 - 2*fp)/(dx*dx);

        if(abs(crv) < solpars.ctol)
            %Curvature is too small along the search direction.
            %Choose to move in negative gradient direction
            p = -hess.A;
            valid = -1;
            if(verbose>2)
                fprintf('Case: no curvature present! crv<%1.6g\n',solpars.ctol);
            end
        else
            %Curvature is present: move towards extremum
            p = -hess.A./crv;
            valid = 0;
            if(verbose>2)
                fprintf('Case: curvature is present. |A| = %1.6g, crv = %1.6g\n',norm(hess.A),crv);
            end
        end
    end
end



return

function hess = int_hessian(fpnt,x,dx,N,fopt)

hess = struct();

f0 = fpnt(x,fopt);
fp = zeros(N,1);
fm = zeros(N,1);
f1 = zeros(N,N);
f2 = zeros(N,N);
f3 = zeros(N,N);
f4 = zeros(N,N);

for I = 1:N
    [xp, xm] = int_xvariant_a(x,dx,I);
    fp(I) = fpnt(xp,fopt);
    fm(I) = fpnt(xm,fopt);
end

for I = 1:N
    for J = I:N
        [x1,x2,x3,x4] = int_variant_b(x,dx,I,J);
        f1(I,J) = fpnt(x1,fopt);
        f2(I,J) = fpnt(x2,fopt);
        f3(I,J) = fpnt(x3,fopt);
        f4(I,J) = fpnt(x4,fopt);
    end
end

dfdx = zeros(N,1);
d2fdxidxj = zeros(N,N);

for I = 1:N
    dfdx(I) = (fp(I) - fm(I))/2/dx;
end

for I = 1:N
    for J = 1:N
        if(J>I)
            d2fdxidxj(I,J) = (f1(I,J)+f3(I,J)-f2(I,J)-f4(I,J))/(4*dx^2);
        elseif(J<I)
            d2fdxidxj(I,J) = (f1(J,I)+f3(J,I)-f2(J,I)-f4(J,I))/(4*dx^2);
        elseif(J==I)
            d2fdxidxj(I,I) = (fp(I) + fm(I) - 2*f0)/dx/dx;
        else
            fprintf('Something is wrong with MATH!\n');
        end
    end
end

hess.H = d2fdxidxj;
hess.A = dfdx;


return

function [xp,xm] = int_xvariant_a(x,dx,I)
    xp = x;
    xm = x;
    if(I>=1&&I<=size(x,1))
        xp(I) = xp(I) + dx;
        xm(I) = xm(I) - dx;
    end
return

function [x1,x2,x3,x4] = int_variant_b(x,dx,I,J)
x1 = x;
x2 = x;
x3 = x;
x4 = x;

if(I>=1&&I<=size(x,1)&&J>=1&&J<=size(x,1)&&I~=J)
    x1(I) = x1(I) + dx;
    x1(J) = x1(J) + dx;
    x2(I) = x2(I) - dx;
    x2(J) = x2(J) + dx;
    x3(I) = x3(I) - dx;
    x3(J) = x3(J) - dx;
    x4(I) = x4(I) + dx;
    x4(J) = x4(J) - dx;
end

return