%Gauss Problem, with p-iteration
%Find initial and final velocities (and therefore orbits), given initial
%and final positions, and a time of flight. 
%
%function [v1_sw, v2_sw, v1_lw, v2_lw] = amsorb_Gaussp(r1,r2,deltat,mu)
%
%r1, r2 - initial and final position column vectors (units: L)
%deltat - time of flight between initial and final position (units: T)
%mu - gravitational constant of central body (units: L^3/T^3)
%Outputs:
%v1_sw - short-way velocity column vector (initial) (units: L/T)
%v2_sw - short-way velocity column vector (final) (units: L/T)
%v1_lw - long-way velocity column vector (initial) (units: L/T)
%v2_lw - long-way velocity column vector (final) (units: L/T)
function [v1_sw, v2_sw, v1_lw, v2_lw, valid] = amsorb_Gaussp_b(r1,r2,deltat,mu)

r1_hat = r1./norm(r1);
r2_hat = r2./norm(r2);
nue1 = acos(dot(r1_hat,r2_hat));
if(nue1 < 0)
    nue1 = nue1 + 2*pi;
end

valid = 1;
if((abs(nue1) <= 1E-6) || (abs(nue1-pi) <= 1E-6)); 
    %Kludge to fix an ill-defined plane of orbit
    %r1_hat = (r1+[1E-6,0,0]')./norm(r1);
    %r2_hat = r2./norm(r2);
    %nue1 = asin(norm(cross(r1_hat,r2_hat)));
    %if(nue1 < 0)
    %    nue1 = nue1 + 2*pi;
    %end
    valid = 0;
end

%I'm basically executing the routine that was canned for me by Bates,
%Mueller, White, Fundamentals of Astrodynamics. Who invented this? How did
%they invent this?

if(valid)
    nue2 = 2*pi - nue1;
    nue_sw = min(nue1,nue2);
    nue_lw = max(nue1,nue2);

    r1m = norm(r1);
    r2m = norm(r2);

    solpars.tol = 1E-4;
    solpars.alph = 0.10;
    solpars.maxiter = 1000;

    %Short way solution
    %1. Solve for p, given deltat,r1m,r2m,dnue,mu
    [p,qf] = int_find_p(deltat,r1m,r2m,nue_sw,mu,solpars);
    if(qf~=1)
        valid = 0;
    end
    %2. Find f,g,fdot,gdot
    g = int_g_func(p,r1m,r2m,nue_sw,mu);
    f = int_f_func(p,r1m,r2m,nue_sw,mu);
    fdot = int_fdot_func(p,r1m,r2m,nue_sw,mu);
    gdot = int_gdot_func(p,r1m,r2m,nue_sw,mu);
    %3. Solve for v1_sw, v2_sw
    v1_sw = 1/g.*(r2 - f.*r1);
    v2_sw = fdot.*r1 + gdot.*v1_sw;

    %Long way solution - solve for p
    %1. Solve for p, given deltat,r1m,r2m,dnue,mu
    [p,qf] = int_find_p(deltat,r1m,r2m,nue_lw,mu,solpars);
    if(qf~=1)
        valid = 0;
    end
    %2. Find f,g,fdot,gdot
    g = int_g_func(p,r1m,r2m,nue_lw,mu);
    f = int_f_func(p,r1m,r2m,nue_lw,mu);
    fdot = int_fdot_func(p,r1m,r2m,nue_lw,mu);
    gdot = int_gdot_func(p,r1m,r2m,nue_lw,mu);
    %3. Solve for v1_sw, v2_sw
    v1_lw = 1/g.*(r2 - f.*r1);
    v2_lw = fdot.*r1 + gdot.*v1_lw;
else
    v1_sw = [0,0,0]';
    v1_lw = [0,0,0]';
    v2_sw = [0,0,0]';
    v2_lw = [0,0,0]';
end

return

function ret = int_k_func(r1m,r2m,dnue)
ret = r1m*r2m*(1-cos(dnue));
return

function ret = int_l_func(r1m,r2m,dnue)
ret = r1m + r2m;
return

function ret = int_m_func(r1m,r2m,dnue)
ret = r1m*r2m*(1+cos(dnue));
return

function ret = int_a_func(p,r1m,r2m,dnue)
k = int_k_func(r1m,r2m,dnue);
l = int_l_func(r1m,r2m,dnue);
m = int_m_func(r1m,r2m,dnue);
ret = m*k*p/((2*m-l^2)*p^2+2*k*l*p-k^2);
return

function [pi, pii] = int_plim_func(r1m,r2m,dnue)
k = int_k_func(r1m,r2m,dnue);
l = int_l_func(r1m,r2m,dnue);
m = int_m_func(r1m,r2m,dnue);
pi = k/(l+sqrt(2*m));
pii = k/(l-sqrt(2*m));
return

function ret = int_f_func(p,r1m,r2m,dnue,mu)
ret = 1-r2m/p*(1-cos(dnue));
return

function ret = int_g_func(p,r1m,r2m,dnue,mu)
ret = r1m*r2m/sqrt(mu*p)*sin(dnue);
return

function ret = int_fdot_func(p,r1m,r2m,dnue,mu)
ret = sqrt(mu/p)*(1-cos(dnue))/sin(dnue)*((1-cos(dnue))/p - 1/r1m -1/r2m);
return

function ret = int_gdot_func(p,r1m,r2m,dnue,mu)
ret = 1-r1m/p*(1-cos(dnue));
return

function ret = int_tof_func(p,r1m,r2m,dnue,mu)
ret = 0;

g = int_g_func(p,r1m,r2m,dnue,mu);
f = int_f_func(p,r1m,r2m,dnue,mu);
fdot = int_fdot_func(p,r1m,r2m,dnue,mu);
a = int_a_func(p,r1m,r2m,dnue);

if(a<0)
    dF = acosh(1-r1m/a*(1-f));
    ret = g + sqrt(-a^3/mu)*(sinh(dF)-dF);
else
    dE = int_arg(1-r1m/a*(1-f), -r1m*r2m*fdot/sqrt(mu*a));
    ret = g + sqrt(a^3/mu)*(dE-sin(dE));
end

return

function th = int_arg(x,y)
%a function, domain (0, 2pi) which returns 
%the angle given by a 2d coordinate x,y
r = sqrt(x.*x+y.*y);
th = 0;

%quadrants
if(y<=x && y>=(-x))
    th = asin(y./r);
    if(th<0) th = th + 2.*pi; end
elseif(y>=x&&y>=-x)
    th = acos(x./r);
elseif(y>=x&&y<=-x)
    th = pi-asin(y./r);
elseif(y<=x&&y<=-x)
    th = 2*pi - acos(x./r);
end

return;

function ret = int_dtdp_func(p,t,r1m,r2m,dnue,mu)
ret = 0;

k = int_k_func(r1m,r2m,dnue);
l = int_l_func(r1m,r2m,dnue);
m = int_m_func(r1m,r2m,dnue);
g = int_g_func(p,r1m,r2m,dnue,mu);
f = int_f_func(p,r1m,r2m,dnue,mu);
fdot = int_fdot_func(p,r1m,r2m,dnue,mu);
a = int_a_func(p,r1m,r2m,dnue);

term1 = -g/2/p;
term2 = -3/2*a*(t-g)*((k^2+(2*m-l^2)*p^2)/(m*k*p^2));
if(a<0)
    dF = acosh(1-r1m/a*(1-f));
    term3 = -sqrt(-a^3/mu)*2*k*sinh(dF)/(p*(k-l*p));
else
    dE = int_arg(1-r1m/a*(1-f), -r1m*r2m*fdot/sqrt(mu*a));
    term3 = sqrt(a^3/mu)*2*k*sin(dE)/(p*(k-l*p));    
end

ret = term1+term2+term3;

return

function [ps,qflag] = int_find_p(deltat,r1m,r2m,dnue,mu,solpars)
%Newton's method solver with analytical solution for dt(p)/dp
%Solves t(p)-deltat = 0 for ps
if(~isfield(solpars,'tol'))
    solpars.tol = 1E-5;
end
if(~isfield(solpars,'maxiter'))
    solpars.maxiter = 1000;
end
if(~isfield(solpars,'alph'))
    solpars.alph = 0.1;
end

%a = int_a_func(p,r1m,r2m,dnue);
[pi, pii] = int_plim_func(r1m,r2m,dnue);

p = 0.5*(pi+pii);
ps = -1;
pold = p;
if(isfield(solpars,'p0')) %Old solution to speed convergence
    p = solpars.p0;
    pold = solpars.p0;
end
deltap = 1E6;
I = 0;
qflag = 0;
while(I<solpars.maxiter && qflag == 0)
    
    tof = int_tof_func(p,r1m,r2m,dnue,mu);
    dtdp = int_dtdp_func(p,tof,r1m,r2m,dnue,mu);
    p = p + solpars.alph*(deltat - tof)/dtdp;
    
    if(1) %Add boundary guards?
        if(p < 0)
            p = 1E-5;
        end
    end
    
    deltap = abs(pold-p);
    if(deltap < solpars.tol)
        qflag = 1;
        ps = p;
    end
    pold = p;
    I = I + 1;
end
if(I>=solpars.maxiter)
    qflag = -1;
    ps = p;
end


return