%Eccentric or hyperbolic anomaly from mean anomaly
%function EF = amsorb_M_to_EF(M,e)
function EF = amsorb_M_to_EF(M,e)

pars = struct();
pars.e = e;
pars.M = M;

x0 = 1E-5;
tol = 1E-6;
dx = 1E-8;
alpha = 0.25;
maxiter = 10000;
verbose = 0;

EF = int_ams_newtons(@int_EF_to_M, x0, pars, tol, alpha, maxiter, dx, verbose);

return

function ret = int_EF_to_M(EF,pars)
e = pars.e;
M = pars.M;

ret = amsorb_EF_to_M(EF,e)-M;

return

function xs = int_ams_newtons(fpnt, x0, pars, tol, alpha, maxiter, dx, verbose)
%Single variable newton's method solver
%Parameters
%fpnt - function pointer taking fpnt(x,pars) and returning error
%x0 - initial guess for x which solves fpnt
%pars - parameter vector passed to fpnt
%tol - relative tolerance for the residual
%alpha - underrelaxation factor to the algorithm - damps divergence
%maxiter - maximum iterations
%dx - scale over which derivative is taken
%verbose - vocalize warnings

%set defaults
if(nargin < 8) verbose = 0; end
if(nargin < 7) dx = 1E-8; end
if(nargin < 6) maxiter = 100000; end
if(nargin < 5) alpha = 0.5; end
if(nargin < 4) tol = 0.001; end
if(nargin < 3) pars = []; end

xs = x0;

I = 1;
exitflag = 0;

if(verbose == 3)
    fprintf('I,x,g(x),dg(x)/dx,err\n');
end


while(I<maxiter && exitflag == 0)
    xsold = xs;
    dfdx = (fpnt(xs+dx,pars)-fpnt(xs-dx,pars))/(2.0*dx);
    fp = fpnt(xs,pars);
    %case - derivative equals zero at this location
    if(dfdx == 0)
        if(verbose==1)
            fprintf('Encountered local extremum, xs = %1.6g, iter = %d - moving randomly\n',xs,I);
        end
        %randomly move within 10 dx to get to a spot that
        %doesn't suck
        xs = xs + 10.0*(2.0*rand()-1.0)*dx;
    else
        %calculate newton's step
        ns = -fp/dfdx;
        %update position
        xs = xs + alpha*ns;
        
        rerr = abs(xs-xsold);
        
        if(verbose==2)
            fprintf('I = %d: x(I) = %1.6g, |x(I)-x(I-1)| = %1.6g,',I,xs,rerr);
            fprintf('g(x) = %1.6g, dg/dx(x) = %1.6g\n',fp,dfdx);
        end
        if(verbose == 3)
            fprintf('%d,%1.5g,%1.5g,%1.5g,%1.5g\n',I,xs,fp,dfdx,rerr);
        end
        
        if(rerr<tol)
            exitflag = 1;
            if(verbose==1)
                fprintf('Success, root found, at xs = %1.6g iteration %d\n',xs,I);
            end
        end
        
    end

    I = I + 1;
end

if(I>=maxiter)
    if(verbose==1)
        fprintf('Failure: Maximum iterations exceeded\n');
    end
end

return