%Universal Time of Flight Problem
%Find r, v (column vectors of position and velocity) at time t-t0, given
%column vectors r0, and vo at time t0
%
%function [r,v] = amsorb_UTOF(r0,v0,tt0,mu)
function [r,v] = amsorb_UTOF(r0,v0,tt0,mu)

oe0 = amsorb_rv_to_oe(r0,v0,mu);
x = int_xgt(tt0,mu,r0,v0,oe0);
f = int_f(tt0,x,r0,oe0,mu);
g = int_g(tt0,x,r0,oe0,mu);
r = r0.*f + v0.*g;
fdot = int_fdot(tt0,x,r0,r,oe0,mu);
gdot = int_gdot(tt0,x,r0,r,oe0,mu);
v = r0.*fdot+v0.*gdot;

return


%%%%%%%%%%%%%%%%%%%%%%
%%Internal Functions%%
%%%%%%%%%%%%%%%%%%%%%%
function ret = int_Cz(z)
if(z>0)
    ret = (1-cos(sqrt(z)))/z;
elseif(z<0)
    ret = (1-cosh(sqrt(-z)))/z;
else
    ret = int_Cz(1E-10);
end
return

function ret = int_Sz(z)
if(z>0)
    ret = (sqrt(z)-sin(sqrt(z)))/sqrt(z^3);
elseif(z<0)
    ret = (sinh(sqrt(-z))-sqrt(-z))/sqrt(-z^3);
else
    ret = int_Sz(1E-10);
end
return

function ret = int_tgx(x,mu,r0,v0,oe0)
z = x^2/oe0.p*(1-oe0.e^2);
term1 = x^3*int_Sz(z);
term2 = dot(r0,v0)/sqrt(mu)*x^2*int_Cz(z);
term3 = norm(r0)*x*(1-z*int_Sz(z));
ret = 1/sqrt(mu)*(term1+term2+term3);
return

function ret = int_xgt(tt0,mu,r0,v0,oe0)
ret = 0;
pars = struct();
pars.mu = mu;
pars.r0 = r0;
pars.v0 = v0;
pars.oe0 = oe0;
pars.tt0 = tt0;

%Newton solver parameters
fpnt = @int_tgxnewt;
tol = 1E-5;
alph = 1;
maxiter = 1000;
dx = 1E-7;
verbose = 0;
x0 = 0;

ret = int_ams_newtons(fpnt,x0,pars,tol,alph,maxiter,dx,verbose);

return

function ret = int_tgxnewt(x,pars)
r0 = pars.r0;
v0 = pars.v0;
oe0 = pars.oe0;
mu = pars.mu;
ret = int_tgx(x,mu,r0,v0,oe0)-pars.tt0;
return

function ret = int_f(tt0,x,r0,oe0,mu)
z = x^2/oe0.p*(1-(oe0.e)^2);
ret = 1-x^2/norm(r0)*int_Cz(z);
return

function ret = int_g(tt0,x,r0,oe0,mu)
z = x^2/oe0.p*(1-(oe0.e)^2);
ret = tt0-x^3/sqrt(mu)*int_Sz(z);
return

function ret = int_fdot(tt0,x,r0,r,oe0,mu)
z = x^2/oe0.p*(1-(oe0.e)^2);
ret = sqrt(mu)/norm(r0)/norm(r)*x*(z*int_Sz(z)-1);
return

function ret = int_gdot(tt0,x,r0,r,oe0,mu)
z = x^2/oe0.p*(1-(oe0.e)^2);
ret = 1-x^2/norm(r)*int_Cz(z);
return

function xs = int_ams_newtons(fpnt, x0, pars, tol, alpha, maxiter, dx, verbose)
%Single variable newton's method solver
%Parameters
%fpnt - function pointer taking fpnt(x,pars) and returning error
%x0 - initial guess for x which solves fpnt
%pars - parameter vector passed to fpnt
%tol - relative tolerance for the residual
%alpha - underrelaxation factor to the algorithm - damps divergence
%maxiter - maximum iterations
%dx - scale over which derivative is taken
%verbose - vocalize warnings

%set defaults
if(nargin < 8) verbose = 0; end
if(nargin < 7) dx = 1E-8; end
if(nargin < 6) maxiter = 100000; end
if(nargin < 5) alpha = 0.5; end
if(nargin < 4) tol = 0.001; end
if(nargin < 3) pars = []; end

xs = x0;

I = 1;
exitflag = 0;

if(verbose == 3)
    fprintf('I,x,g(x),dg(x)/dx,err\n');
end


while(I<maxiter && exitflag == 0)
    xsold = xs;
    dfdx = (fpnt(xs+dx,pars)-fpnt(xs-dx,pars))/(2.0*dx);
    fp = fpnt(xs,pars);
    %case - derivative equals zero at this location
    if(dfdx == 0)
        if(verbose==1)
            fprintf('Encountered local extremum, xs = %1.6g, iter = %d - moving randomly\n',xs,I);
        end
        %randomly move within 10 dx to get to a spot that
        %doesn't suck
        xs = xs + 10.0*(2.0*rand()-1.0)*dx;
    else
        %calculate newton's step
        ns = -fp/dfdx;
        %update position
        xs = xs + alpha*ns;
        
        rerr = abs(xs-xsold);
        
        if(verbose==2)
            fprintf('I = %d: x(I) = %1.6g, |x(I)-x(I-1)| = %1.6g,',I,xs,rerr);
            fprintf('g(x) = %1.6g, dg/dx(x) = %1.6g\n',fp,dfdx);
        end
        if(verbose == 3)
            fprintf('%d,%1.5g,%1.5g,%1.5g,%1.5g\n',I,xs,fp,dfdx,rerr);
        end
        
        if(rerr<tol)
            exitflag = 1;
            if(verbose==1)
                fprintf('Success, root found, at xs = %1.6g iteration %d\n',xs,I);
            end
        end
        
    end

    I = I + 1;
end

if(I>=maxiter)
    if(verbose==1)
        fprintf('Failure: Maximum iterations exceeded\n');
    end
end

return
