%Returns a standard keplerian orbit element set, given the orbit elements
%sets supplied by JPL for planets and moons.
%
%Requires: kepler_rate object, julian calendar date 
function oe = amsorb_krate_to_oe(krate,JD,mu)
oe = struct();

if(~strcmp(krate.type,'kepler_rate'))
    fprintf('Warning! Object type is not kepler_rate. Is %s.',krate.type);
    fprintf(' This function expects certain types of keplerian elements and element rates provided in the form given by JPL\n');
end

if(~isfield(krate,'epoch'))
    %No epoch provided - all orbital elements must be assumed to be
    %constant. Epoch is assumed to be J2000
    epoch = 2451545.0; %J2000 epoch in JD (Julian calendar days)
    t0 = epoch*24*3600; %"julian seconds"
    t1 = JD*24*3600;
    T = t1-t0;  %Seconds since epoch
    
    oe = struct();
    a = krate.a;
    e = krate.e;
    inc = krate.inc;
    Omega = krate.Omega;
    lperi = krate.lperi;
    L = krate.L;
    lomega = lperi-Omega;
    M = L-lperi;
    E = amsorb_M_to_EF(M,e);
    nue = amsorb_EF_to_nue(EF,e);
    
    oe.orbitvalid = 1;
    oe.inc = inc;
    oe.Omega = Omega;
    oe.Omegavalid = 1;
    oe.lomega = lomega;
    oe.nue = nue;
    oe.e = e;
    oe.a = a;

elseif(~isfield(krate,'epochmin')||~isfield(krate,'epochmax'))
    %No epoch boundaries provided: The provided rate data will be assumed
    %valid for all time
    epoch = krate.epoch;
    
    t0 = epoch*24*3600; %"julian seconds"
    t1 = JD*24*3600;
    T = t1-t0;  %Seconds since epoch
    
    oe = struct();
    a = krate.a+krate.adot*T;
    e = krate.e+krate.edot*T;
    inc = krate.inc+krate.incdot*T;
    Omega = krate.Omega+krate.Omegadot*T;
    lperi = krate.lperi+krate.lperidot*T;
    L = krate.L+krate.Ldot*T;
    lomega = lperi-Omega;
    M = L-lperi;
    E = amsorb_M_to_EF(M,e);
    nue = amsorb_EF_to_nue(E,e);
    
    oe.orbitvalid = 1;
    oe.inc = inc;
    oe.Omega = Omega;
    oe.Omegavalid = 1;
    oe.lomega = lomega;
    oe.nue = nue;
    oe.e = e;
    oe.a = a;
    
else
    %Epoch boundaries are provided. If t1<tmin, set orbit elements to first
    %valid values and propagate backwards.
    %if t1>tmax, set orbit elements to last values and propagate forwards.
    %if tmin<t1<tmax, use the orbit element rates provided
    epoch = krate.epoch;
    epochmin = krate.epochmin;
    epochmax = krate.epochmax;
    t0 = epoch*3600*24;
    tmin = epochmin*3600*24;
    tmax = epochmax*3600*24;
    t1 = JD*3600*24;
    T = t1-t0;
    Tmin = tmin-t0;
    Tmax = tmax-t0;
    
    if(tmin<=t1 && t1<=tmax)
        oe = struct();
        a = krate.a+krate.adot*T;
        e = krate.e+krate.edot*T;
        inc = krate.inc+krate.incdot*T;
        Omega = krate.Omega+krate.Omegadot*T;
        lperi = krate.lperi+krate.lperidot*T;
        L = krate.L+krate.Ldot*T;
        lomega = lperi-Omega;
        M = L-lperi;
        E = amsorb_M_to_EF(M,e);
        nue = amsorb_EF_to_nue(E,e);

        oe.orbitvalid = 1;
        oe.inc = inc;
        oe.Omega = Omega;
        oe.Omegavalid = 1;
        oe.lomega = lomega;
        oe.nue = nue;
        oe.e = e;
        oe.a = a;
        
    elseif(t1<tmin)
        oe = struct();
        a = krate.a+krate.adot*Tmin;
        e = krate.e+krate.edot*Tmin;
        inc = krate.inc+krate.incdot*Tmin;
        Omega = krate.Omega+krate.Omegadot*Tmin;
        lperi = krate.lperi+krate.lperidot*Tmin;
        L = krate.L+krate.Ldot*Tmin;
        lomega = lperi-Omega;
        M = L-lperi;
        E = amsorb_M_to_EF(M,e);
        nue = amsorb_EF_to_nue(E,e);

        oe1.orbitvalid = 1;
        oe1.inc = inc;
        oe1.Omega = Omega;
        oe1.Omegavalid = 1;
        oe1.lomega = lomega;
        oe1.nue = nue;
        oe1.e = e;
        oe1.a = a;
        
        oe = amsorb_oe_to_oe_t(oe1,t1-tmin,mu);
        
    else
        oe = struct();
        a = krate.a+krate.adot*Tmax;
        e = krate.e+krate.edot*Tmax;
        inc = krate.inc+krate.incdot*Tmax;
        Omega = krate.Omega+krate.Omegadot*Tmax;
        lperi = krate.lperi+krate.lperidot*Tmax;
        L = krate.L+krate.Ldot*Tmax;
        lomega = lperi-Omega;
        M = L-lperi;
        E = amsorb_M_to_EF(M,e);
        nue = amsorb_EF_to_nue(E,e);

        oe1.orbitvalid = 1;
        oe1.inc = inc;
        oe1.Omega = Omega;
        oe1.Omegavalid = 1;
        oe1.lomega = lomega;
        oe1.nue = nue;
        oe1.e = e;
        oe1.a = a;
        
        oe = amsorb_oe_to_oe_t(oe1,t1-tmax,mu);
    end

end



return