%Orbit to orbit intercept problem
%Return the minimum delta-V (short or long) patched conic orbit between one
%orbiting object and another.
%Inputs:
%oe1 - initial orbit element set
%epoch1 - JD epoch of initial orbit data
%t1 - time of departure from initial orbit (JD)
%oe2 - target orbit element set
%epoch2 - JD epoch of target orbit elements (JD)
%t2 - time of arrival to final orbit
%mu - gravitational constant of central body (m^3/sec^2)
%
%Outputs:
%r1 - column vector of transfre orbit initial position (m)
%v1 - column vector of transfer orbit initial velocity (m/sec)
%r2 - column vector of transfer orbit final position   (m)
%v2 - column vector of transfer orbit final velocity   (m/sec)
%is_sw - true if trajectory is "short way" in terms of delta-nue. false if
%        it is long-way
%dv1 - delta-V to transfer from initial to transfer orbit (m/sec)
%dv2 - delta-V to transfer from transfer to final orbit   (m/sec)
function [r1,v1,r2,v2,is_sw,dv1,dv2] = amsorb_oe_to_oe_intercept(oe1,epoch1,t1,oe2,epoch2,t2,mu)

tt1 = (t1-epoch1)*3600*24;
tt2 = (t2-epoch2)*3600*24;

oe1_t1 = amsorb_oe_to_oe_t(oe1,tt1,mu);
oe2_t2 = amsorb_oe_to_oe_t(oe2,tt2,mu);
tof = (t2-t1)*3600*24;
[r1,v01] = amsorb_oe_to_rv(oe1_t1,mu);
[r2,v02] = amsorb_oe_to_rv(oe2_t2,mu);
[v1_sw,v2_sw,v1_lw,v2_lw,valid] = amsorb_Gaussp_b(r1,r2,tof,mu);

if(valid)
    dv1_sw = v1_sw-v01;
    dv1_swm = norm(dv1_sw);
    dv2_sw = v02 - v2_sw;
    dv2_swm = norm(dv2_sw);
    dv1_lw = v1_lw-v01;
    dv1_lwm = norm(dv1_lw);
    dv2_lw = v02 - v2_lw;
    dv2_lwm = norm(dv2_lw);
    
    if((dv1_swm+dv2_swm)<(dv1_lwm+dv2_lwm))
        is_sw = 1;
        dv1 = dv1_swm;
        dv2 = dv2_swm;
        v1 = v1_sw;
        v2 = v2_sw;
    else
        is_sw = 0;
        dv1 = dv1_lwm;
        dv2 = dv2_lwm;
        v1 = v1_lw;
        v2 = v2_lw;
    end
else
    is_sw = -1;
    v1 = [0,0,0]'; v2=[0,0,0]'; dv1 = -1; dv2 = -1;
end
    
return;
