%Instantaneous orbit elements to radius and velocity vector
%Minimum orbit elements required
%One of the following:
%   oe.p - orbit semi-latus rectum
%   oe.a - orbit semi-major axis
%oe.e - eccentricity
%oe.inc - inclination
%oe.Omega - longitude of ascending node
%oe.lomega - argument of periapsis
%oe.nue - semi major
%
%mu - L^3/T^2 Central body gravity constant
function [r,v] = amsorb_oe_to_rv(oe,mu)

r = [0,0,0]';
v = [0,0,0]';

[oe,val] = int_orbit_validate(oe); %validate and fill in oe from what is given
if(val == 0)
    fprintf('Error! Supplied orbit element struct is invalid.\n');
else
   inc = oe.inc;
   Om = oe.Omega;
   A = [1,0,0;0,cos(inc),-sin(inc);0,sin(inc),cos(inc)];
   B = [cos(Om),-sin(Om),0;sin(Om),cos(Om),0;0,0,1];
   C = B*A;
   un = C(:,1);     %line of nodes
   unp = C(:,2);    %complement orbit plane vector
   uh = C(:,3);     %angular momentum unit vector
   
   lom = oe.lomega;
   nue = oe.nue;
   
   ue = cos(lom).*un + sin(lom).*unp;   %eccentricity unit vector
   up = -sin(lom).*un + cos(lom).*unp;  %semi-latus-rectum unit vector
   ur = cos(lom+nue).*un + sin(lom+nue).*unp;
   unue = cross(uh,ur);
   
   rmag = oe.p/(1+oe.e*cos(nue));
   r = rmag.*ur;
   h = sqrt(mu*oe.p);
   v1 = h/rmag^2*(rmag*oe.e*sin(nue)/(1+oe.e*cos(nue)));
   v2 = h/rmag^2*(rmag);
   v = v1.*ur + v2.*unue; 
    
end
return

function [oe,val] = int_orbit_validate(oe)
val = 1;
if(~isfield(oe,'e'))
    fprintf('OE missing e\n');
    val = 0;
end
if(~isfield(oe,'inc'))
    fprintf('OE missing inc\n');
    val = 0;
end
if(~isfield(oe,'Omega'))
    fprintf('OE missing Omega\n');
    val = 0;
end
if(~isfield(oe,'lomega'))
    fprintf('OE missing lomega\n');
    val = 0;
end
if(~isfield(oe,'nue'))
    fprintf('OE missing nue\n');
    val = 0;
end
if(isfield(oe,'p')&&val==1)
    oe.a = oe.p/(1-oe.e^2);
else
    if(isfield(oe,'a')&&val==1)
        oe.p = oe.a*(1-oe.e^2);
    else
        val = 0;
        fprintf('OE missing p and a\n');
    end
end

return