%Porkchop plot problem
%Returns an array of delta-Vs for constructing a porkchop plot
%Inputs:
%oe1 - initial orbit element set
%epoch1 - JD epoch of initial orbit data
%t1_1 - (JD) - first departure time
%t1_2 - (JD) - last departure time
%Nt1 - number of departure time trials
%oe2 - target orbit element set
%epoch2 - JD epoch of target orbit elements (JD)
%t2_1 - first arrival time (JD)
%t2_2 - last arrival time (JD)
%Nt2 - number of arrival time trials
%mu - gravitational constant of central body (m^3/sec^2)
%
%Outputs:
%t1 - Nt1 x Nt2 array of arrival times
%t2 - Nt1 x Nt2 array of departure times
%r1{Nt1,Nt2} - column vector of transfre orbit initial position (m)
%v1{Nt1,Nt2} - column vector of transfer orbit initial velocity (m/sec)
%r2{Nt1,Nt2} - column vector of transfer orbit final position   (m)
%v2{Nt1,Nt2} - column vector of transfer orbit final velocity   (m/sec)
%is_sw(Nt1,Nt2) - true if trajectory is "short way" in terms of delta-nue. false if
%        it is long-way
%dv1(Nt1,Nt2) - delta-V to transfer from initial to transfer orbit (m/sec)
%dv2(Nt1,Nt2) - delta-V to transfer from transfer to final orbit   (m/sec)
function [t1,t2,r1,v1,r2,v2,is_sw,dv1,dv2] = amsorb_porkchopproblem(oe1,epoch1,t1_1,t1_2,Nt1,oe2,epoch2,t2_1,t2_2,Nt2,mu,verbose)

if(nargin<12)
    verbose = 0;
end

%initialize
tt1 = linspace(t1_1,t1_2,Nt1);
tt2 = linspace(t2_1,t2_2,Nt2);
t1 = zeros(Nt1,Nt2);
t2 = zeros(Nt1,Nt2);
for I = 1:Nt1
    for J = 1:Nt2
        t1(I,J) = tt1(I);
        t2(I,J) = tt2(J);
        r1{I,J} = [0,0,0]';
        r2{I,J} = [0,0,0]';
        v1{I,J} = [0,0,0]';
        v2{I,J} = [0,0,0]';
    end
end
is_sw = ones(Nt1,Nt2).*-2;
dv1 = ones(Nt1,Nt2).*-2;
dv2 = ones(Nt1,Nt2).*-2;

%For each time pair, solve the orbit to orbit intercept problem

for I = 1:Nt1
    for J = 1:Nt2
        ttt1 = t1(I,J);
        ttt2 = t2(I,J);
        if(ttt2>ttt1)
            %function [r1,v1,r2,v2,is_sw,dv1,dv2] = amsorb_oe_to_oe_intercept(oe1,epoch1,t1,oe2,epoch2,t2)
            [rr1,vv1,rr2,vv2,is_sww,ddv1,ddv2] = amsorb_oe_to_oe_intercept(oe1,epoch1,ttt1,oe2,epoch2,ttt2,mu);
            r1{I,J} = rr1;
            v1{I,J} = vv1;
            r2{I,J} = rr2;
            v2{I,J} = vv2;
            is_sw(I,J) = is_sww;
            dv1(I,J) = ddv1;
            dv2(I,J) = ddv2;
        end
    end
    
    if(verbose == 1)
        fprintf('%1.2f %% complete\n',I/Nt1*100);
    end
end

return