%Orbit elements from r, v vectors
%Also, give mu (L^3/T^2) for orbit determination problem
%function oe = amsorb_rv_to_oe(r,v,mu)
%Inputs:
%r - [rx;ry;rz] - column vector (L units)
%v - column vector (L/T units)
%mu - gravity constant of central body (L^3/T^2)
%Outputs:
%oe - struct containing keplerian orbit elements, and flags for angle validation
%    oe.orbitvalid = 0;  %validity of orbit? Is r // to v?
%    oe.inc = 0;         %inclination angle
%    oe.Omega = 0;       %longitude of ascending node
%    oe.Omegavalid = 0;  %validity of long of ascending node (is h // to K)?
%    oe.lomega = 0;      %argument of periapsis
%    oe.lomegavalid = 0; %validity of arg of periapsis: Is orbit circular?
%    oe.nue = 0;         %true anomaly
%    oe.e = 0;           %eccentricity magnitude
%    oe.p = 0;           %semi-latus rectum
%    oe.a = 0;           %semimajor axis
%    oe.rp = 0;          %radius at periapsis
%    oe.ra = 0;          %radius at apoapsis
%
%Note: Orbit elements are returned for a frame oriented along vector's
%direction. [1,0,0], [0,1,0], [0,0,1] for x,y,z x-z equatorial plane
function oe = amsorb_rv_to_oe(r,v,mu)

r = int_make_colvect(r);
v = int_make_colvect(v);

%oe = struct();
oe = int_orbit_defaults(); %set all values of this struct to default before beginning

I = [1,0,0]';
J = [0,1,0]';
K = [0,0,1]';

h = cross(r,v); %Angular momentum vector
if(norm(h)>0)
    uh = h./norm(h);
    oe.orbitvalid = 1; 
else
    uh = [1,0,0];
    oe.orbitvalid = 0; %Not really an orbit if rxv = 0
end

oe.inc = acos(dot(uh,K)); %inclination angle

n = cross(K,h); %Line of nodes, ascending node

if(norm(n)>0)
    un = n./norm(n);
    oe.Omega = arg_vect(I,J,un);
    oe.Omegavalid = 1;
else
    n = [1,0,0]; %h perpindicular to x,y plane
    un = [1,0,0];
    oe.Omega = 0;
    oe.Omegavalid = 0;
end

nprime = cross(uh,un); %complement vector to line of nodes, defining orbit plane
unprime = nprime/norm(nprime);

e = 1/mu*((dot(v,v)-mu/norm(r)).*r - (dot(r,v)).*v); %eccentricity vector

oe.e = norm(e); %eccentricity
oe.p = norm(h)^2/mu; %I don't know what this one is called, but is well defined, even in parabolic orbits
oe.a = oe.p/(1-oe.e^2); %semimajor axis

if(norm(e)>0)
    ue = e./norm(e);
    oe.lomega = arg_vect(un,unprime,ue); %argument of periapsis
    oe.lomegavalid = 1;
else
    ue = un; %circular orbit - e set to n by default
    oe.lomega = arg_vect(un,unprime,ue);
    oe.lomegavalid = 0;
end

p1 = cross(h,ue); %semi-latus rectum
up = p1/norm(p1); %unit vector in perifocal direction to complete (e,p,h) perifocal frame

oe.nue = arg_vect(ue,up,r); %true anomaly

oe.rp = oe.p/(1+oe.e);          %radius at periapsis
oe.ra = oe.p/(1-oe.e);          %radius at apoapsis

return

%%%%%%%%%%%%%%%%%%%%%
%Internal Functions%%
%%%%%%%%%%%%%%%%%%%%%

function oe = int_orbit_defaults()
    oe = struct();
    oe.orbitvalid = 0;  %validity of orbit? Is r // to v?
    oe.inc = 0;         %inclination angle
    oe.Omega = 0;       %longitude of ascending node
    oe.Omegavalid = 0;  %validity of long of ascending node (is h // to K)?
    oe.lomega = 0;      %argument of periapsis
    oe.lomegavalid = 0; %validity of arg of periapsis: Is orbit circular?
    oe.nue = 0;         %true anomaly
    oe.e = 0;           %eccentricity magnitude
    oe.p = 0;           %semi-latus rectum
    oe.a = 0;           %semimajor axis
    oe.rp = 0;          %radius at periapsis
    oe.ra = 0;          %radius at apoapsis
return

function th = int_arg(x,y)
%a function, domain (0, 2pi) which returns 
%the angle given by a 2d coordinate x,y
r = sqrt(x.*x+y.*y);
th = 0;
%quadrants
if(y<=x && y>=(-x))
    th = asin(y./r);
    if(th<0) th = th + 2.*pi; end
elseif(y>=x&&y>=-x)
    th = acos(x./r);
elseif(y>=x&&y<=-x)
    th = pi-asin(y./r);
elseif(y<=x&&y<=-x)
    th = 2*pi - acos(x./r);
end

return

%Doesn't matter what form a vector comes in as, make it a column vector
function y=int_make_colvect(x)
a = size(x);
if(a(1)==1)
    y = x';
else
    y = x;
end
return

%Returns the angle of the component of vector ev in the oriented plane
%defined by xv and yv. The angle is measured counterclockwise from xv,
%towards yv, then around.
%
%xv,yv,ev are meant to be 3 dimensional column vectors
%function theta = arg_vect(xv,yv,ev)
function theta = arg_vect(xv,yv,ev)
xv = int_make_colvect(xv);
yv = int_make_colvect(yv);
ev = int_make_colvect(ev);
theta = 0;
xvv = xv./norm(xv);
yvvp = yv./norm(yv);
yvv = yvvp - (yvvp'*xvv)*xvv; %orthonormalize if it isn't already
x = ev'*xvv;
y = ev'*yvv;

theta = int_arg(x,y);

return


