%Calculates positions, velocities, and orbit elements for all objects in a
%solarsystem tree for a given epoch time.
%Generates and populates the eprop (epoch property) struct for each object.
%
%function solsys = amsorb_solsysepochcalc(solsys,JD)
function solsys = amsorb_solsysepochcalc(solsys,JD)

%Calculate orbit elements at epoch (if applicable)
for I = 1:size(solsys.objs,2)
    eprop = struct();
    eprop.epoch = JD;
    if(strcmp(solsys.objs{I}.orbit.type,'kepler_rate'))
        mu_parent = solsys.objs{I}.prop.mu_parent;
        krate = solsys.objs{I}.orbit;
        oe = amsorb_krate_to_oe(krate,JD,mu_parent);
        eprop.oe = oe;
    end
    if(strcmp(solsys.objs{I}.orbit.type,'kepler_JPLmoon'))
        mu_parent = solsys.objs{I}.prop.mu_parent;
        kjplm = solsys.objs{I}.orbit;
        oe = amsorb_kjplm_to_oe(kjplm,JD,mu_parent);
        eprop.oe = oe;
        %function oe = amsorb_kjplm_to_oe(kjplm,JD,mu)
    end
    solsys.objs{I}.eprop = eprop;
end

%Start at root and iterate through object tree
%Calculate each object's position and velocity
%then
%calculate each object's children's position and velocity
solsys = int_calcposvel(solsys,solsys.rootobj);

%calculate each object's sphere of influence (SOI) relative to parent
%object.
for I = 1:size(solsys.objs,2)
    obj = solsys.objs{I};
    if(obj.parent~=-1)
        pobj = solsys.objs{obj.parent};
        m1 = pobj.prop.mass;
        m2 = obj.prop.mass;
        oe = obj.eprop.oe;
        oe = amsorb_calcoeaux(oe);
        rsp = oe.p/(1+oe.e*cos(oe.nue));
        rsoi = rsp*(m2/m1)^(2/5);
        solsys.objs{I}.eprop.rsoi = rsoi;
    end
end


return

%Start at root and iterate through object tree
%Calculate each object's position and velocity
%then
%calculate each object's children's position and velocity
function solsys = int_calcposvel(solsys,ind)

if(solsys.objs{ind}.parent == -1)
    %no parent exists
    %calc initial position and velocity
    if(strcmp(solsys.objs{ind}.orbit.type,'fixed'))
        r = solsys.objs{ind}.orbit.location;
        v = [0,0,0]';
        solsys.objs{ind}.eprop.r = r;
        solsys.objs{ind}.eprop.v = v;
    else
        fprintf('Warning: Root objects orbit type is not fixed\n');
        r = [0,0,0];
        v = [0,0,0];
        solsys.objs{ind}.eprop.r = r;
        solsys.objs{ind}.eprop.v = v;
    end
else
    %find parent's position and velocity
    parent = solsys.objs{ind}.parent;
    rpar = solsys.objs{parent}.eprop.r;
    vpar = solsys.objs{parent}.eprop.v;
    %calculate relative position and velocity from orbit elements
    if(strcmp(solsys.objs{ind}.orbit.type,'fixed'))
        %not really physical - maintains fixed position relative to parent
        rrel = solsys.objs{ind}.orbit.location;
        vrel = [0,0,0]';
    else
        %there should exist an oe in eprop
        oe = solsys.objs{ind}.eprop.oe;
        mu = solsys.objs{ind}.prop.mu_parent;
        [rrel,vrel] = amsorb_oe_to_rv(oe,mu);
    end
    r = rrel+rpar;
    v = vrel+vpar;
    solsys.objs{ind}.eprop.r = r;
    solsys.objs{ind}.eprop.v = v;
    solsys.objs{ind}.eprop.rrel = rrel;
    solsys.objs{ind}.eprop.vrel = vrel;
end

%Call this function for all children
for I = 1:size(solsys.objs{ind}.children,2)
    chind = solsys.objs{ind}.children{I};
    solsys = int_calcposvel(solsys,chind);
end

return