%Test using conjugate gradient unconstrained optimization to minimize
clc; clear all;
path(path,'../ConstrainedOpt_Matlab');

%Constants
G = 6.67408E-11; %N-m^2/kg^2
AU = 149597870700; %m in an AU
cty = 100*365.25*24*3600; %sec in a century
rad = pi/180; %conv deg to radians
day = 24*3600; %sec in a day

Nps = 20; %Number of trajectory control points

%I. Set Departure and Arrival Dates
JD0 = ams_juliandate(2022,9,27,0,0,0);
JDf = ams_juliandate(2023,2,15,0,0,0);
t0 = JD0*3600*24;
tf = JDf*3600*24;

%II. Get Solsystem Model
SS = amsorb_generatesolarsys();
mu_sol = SS.objs{1}.prop.mu;

%III. Get Initial and Final Coordinates
SS0 = amsorb_solsysepochcalc(SS,JD0);
SSf = amsorb_solsysepochcalc(SS,JDf);

r0 = SS0.objs{4}.eprop.r;
v0 = SS0.objs{4}.eprop.v;
rf = SSf.objs{5}.eprop.r;
vf = SSf.objs{5}.eprop.v;

%IV. Generate TOP Initial Guess Trajectory (Cubic Spline)
%initguess = amsorb_top_generateICs();
initguess = amsorb_top_generateICs(r0,v0,t0,rf,vf,tf,Nps);

if(0)
    %Plot the initial guess
    rc = initguess.rc;
    for I = 1:size(rc,2)
        xc(I) = rc{I}(1)/AU;
        yc(I) = rc{I}(2)/AU;
        zc(I) = rc{I}(3)/AU;
    end
    figure(1);
    plot3(xc,yc,zc,'.');
    axis equal
end


%V. CGLS Solver Parameters
solpars = struct();
solpars.verbose = 2; %Display progress
solpars.maxiter = 500;
solpars.dx = 5000;
solpars.vtol = 1E-12;
solpars.ctol = 1E-30;
solpars.use_reltol = 0;
solpars.reltol = 1E-10;

%solpars.fdesc = 1;
%solpars.use_alphastep = 1;
%solpars.alpha = 0.25;

fopt = struct();
fopt.Np = Nps;
fopt.mu = mu_sol;
fopt.rc0 = initguess.rc{1};
fopt.rc1 = initguess.rc{2};
fopt.rcNm1 = initguess.rc{end-1};
fopt.rcN = initguess.rc{end};
fopt.tl = initguess.t;
x0 = amsorb_top_wrappars(initguess.rc,Nps);
Ndim = size(x0,1);
deltaV0 = amsorb_top_fdv_v2(x0,fopt); %m/sec delta-V of initial solution
obj0 = amsorb_top_fobj_v2(x0,fopt);   %(m^2/sec^2)*sec objective function

fprintf('Initial solution: dV = %1.6g km/sec, obj = %1.6g m^2/sec\n',deltaV0/1000,obj0);

%VI. Optimize TOP Initial Guess
[xf,val] = amsopt_cgls(@amsorb_top_fobj_v2,x0,fopt,Ndim,solpars);
%[xf,val] = amsopt_newton_bfgh(@amsorb_top_fobj_v2,x0,Ndim,fopt,solpars);
%[xf,val] = amsopt_newton_v2(@amsorb_top_fobj_v2,x0,Ndim,fopt,solpars);

rcf = amsorb_top_unwrappars(xf,Nps,fopt);


%VII. Generate Statistics for Trajectory
deltaVf = amsorb_top_fdv_v2(xf,fopt); %m/sec delta-V of initial solution
objf = amsorb_top_fobj_v2(xf,fopt);   %(m^2/sec^2)*sec objective function
fprintf('Initial solution: dV = %1.6g km/sec, obj = %1.6g m^2/sec^3\n',deltaV0/1000,obj0);
fprintf('Final solution: dV = %1.6g km/sec, obj = %1.6g m^2/sec^3\n',deltaVf/1000,objf);

stats = amsorb_top_solstat_v2(xf,fopt);

amx = -1;
for I = 1:size(stats.ra,2)
    if(norm(stats.au{I})>amx)
        amx = norm(stats.au{I});
    end
end
fprintf('Max acceleration: %1.6g m/sec^2, %1.6g mm/sec^2\n',amx,amx*1000);

    

%VIII. Plot Trajectory
if(1)
    figure(1);
    clf; hold on;
    %Plot the initial guess
    rc = initguess.rc;
    for I = 1:size(rc,2)
        xc(I) = rc{I}(1);
        yc(I) = rc{I}(2);
        zc(I) = rc{I}(3);
    end
    figure(1);
    clf; hold on;
    plot3(xc,yc,zc,'b.');
    plot3(xc,yc,zc,'b-');
    axis equal
    
    %Plot the final guess
    for I = 1:size(rc,2)
        xcf(I) = rcf{I}(1);
        ycf(I) = rcf{I}(2);
        zcf(I) = rcf{I}(3);
    end
    plot3(xcf,ycf,zcf,'r.');
    plot3(xcf,ycf,zcf,'r-');
    axis equal
    
    %Draw the orbits of the planets
    plot3(0,0,0,'o','Color',[0.3,0.3,0]);
    ams_draworbit(SS0.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SSf.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SS0.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    ams_draworbit(SSf.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    
    %Draw thrust vectors
    for I = 1:size(stats.ra,2)
        ams_drawarrow(stats.ra{I},stats.au{I}.*5E12,[1,0,0]);
        ams_drawarrow(stats.ra{I},(stats.ac{I}-stats.au{I}).*5E12,[0,1,0]);  
    end
    
end
