clc; clear all;


G = 6.67408E-11; %N-m^2/kg^2
AU = 149597870700; %m in an AU
cty = 100*365.25*24*3600; %sec in a century
rad = pi/180; %conv deg to radians
day = 24*3600; %sec in a day

%I. Set Departure and Arrival Dates
JD0 = ams_juliandate(2022,9,27,0,0,0);
JDf = ams_juliandate(2023,4,15,0,0,0);
t0 = JD0*3600*24;
tf = JDf*3600*24;

%II. Get Solsystem Model
SS = amsorb_generatesolarsys();
mu_sol = SS.objs{1}.prop.mu;

%III. Get Initial and Final Coordinates
SS0 = amsorb_solsysepochcalc(SS,JD0);
SSf = amsorb_solsysepochcalc(SS,JDf);

r0 = SS0.objs{4}.eprop.r;
v0 = SS0.objs{4}.eprop.v;
rf = SSf.objs{5}.eprop.r;
vf = SSf.objs{5}.eprop.v;

%IV. Setup problem parameters
ppars.t0 = t0;
ppars.tf = tf;
ppars.r0 = r0;
ppars.v0 = v0;
ppars.rf = rf; %the target, not necessarily what will be acheived
ppars.vf = vf; %the target, not necessarily what will be acheived
ppars.mu = mu_sol;
ppars.Nt = 100; %number of time increments to use for integration
ppars.Nazel = 3; %number of azimuth/elevation control points to use
ppars.T = 0.005; %thrust magnitude (acceleration at initial mass, m/sec^2)
ppars.Isp = 4000; %specific impulse (sec)
ppars.m0 = 1; %mass (fraction of initial mass at start of arc)

%V. Get initial optimization parameter struct
optpars = ams_top2_gen_ics(ppars);

%VI. Run problem
outs = amsorb_top2_traj(optpars,ppars);

%analyze final errors
rf2 = outs.rlst{end};
vf2 = outs.vlst{end};
fprintf('r error (m): %1.6g m %1.6g%%\n',norm(rf-rf2),norm(rf-rf2)/norm(rf)*100);
fprintf('v error (m/sec): %1.6g m/sec %1.6g%%\n',norm(vf-vf2),norm(vf-vf2)/norm(vf)*100);

z1 = amsorb_top2_unspool_z(optpars,t0,tf);
op2 = amsorb_top2_spool_z(z1,t0,tf);

%VII. Plot trajectory
for I = 1:size(outs.rlst,2)
    r = outs.rlst{I};
    xx(I) = r(1);
    yy(I) = r(2);
    zz(I) = r(3);
end

figure(1);
clf; hold on;
    ams_draworbit(SS0.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SSf.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SS0.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    ams_draworbit(SSf.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    plot3(0,0,0,'ko');
    
    h = plot3(xx,yy,zz);
    set(h,'Color',[0,1,0]);
    
    axis equal;
        