clc; clear all;
path(path,'../ConstrainedOpt_Matlab');

G = 6.67408E-11; %N-m^2/kg^2
AU = 149597870700; %m in an AU
cty = 100*365.25*24*3600; %sec in a century
rad = pi/180; %conv deg to radians
day = 24*3600; %sec in a day

%I. Set Departure and Arrival Dates
JD0 = ams_juliandate(2022,9,27,0,0,0);
JDf = ams_juliandate(2023,11,15,0,0,0);
t0 = JD0*3600*24;
tf = JDf*3600*24;

%II. Get Solsystem Model
SS = amsorb_generatesolarsys();
mu_sol = SS.objs{1}.prop.mu;

%III. Get Initial and Final Coordinates
SS0 = amsorb_solsysepochcalc(SS,JD0);
SSf = amsorb_solsysepochcalc(SS,JDf);

r0 = SS0.objs{4}.eprop.r;
v0 = SS0.objs{4}.eprop.v;
rf = SSf.objs{5}.eprop.r;
vf = SSf.objs{5}.eprop.v;

%IV. Setup problem parameters
ppars.t0 = t0;
ppars.tf = tf;
ppars.r0 = r0;
ppars.v0 = v0;
ppars.rf = rf; %the target, not necessarily what will be acheived
ppars.vf = vf; %the target, not necessarily what will be acheived
ppars.mu = mu_sol;
ppars.Nt = 200; %number of time increments to use for integration
ppars.Nazel = 3; %number of azimuth/elevation control points to use
ppars.T = 0.0005; %thrust magnitude (acceleration at initial mass, m/sec^2)
ppars.Isp = 4000; %specific impulse (sec)
ppars.m0 = 1; %mass (fraction of initial mass at start of arc)

%V. Get initial optimization parameter struct
optpars = ams_top2_gen_ics(ppars);
z0 = amsorb_top2_unspool_z(optpars,t0,tf);

%VI. Optimize optimization parameters with constrained optimization problem
%VI. Constrained Newton Optimization Paramters
solpars = struct();
solpars.verbose = 2; %Display progress
solpars.maxiter = 80;
solpars.dx = 1E-5;
solpars.vtol = 1E-15;
solpars.ctol = 1E-30;
solpars.use_reltol = 0;
solpars.reltol = 1E-15;
solpars.use_alphastep = 1;
solpars.alpha = 0.2;

ppars.cnstscale = 1E2;
fobj = @amsorb_top2_fobj_2b;

J0 = fobj(z0,ppars);
out0 = amsorb_top2_traj(optpars,ppars);

%Initial solution statistics
fprintf('Initial solution stats:\n');
fprintf('dV = %1.6g m/sec J2 = %1.6g m^2/sec^3\n', out0.dV, out0.J2);
re0 = norm(out0.rlst{end}-rf);
ve0 = norm(out0.vlst{end}-vf);
fprintf('R_error = %1.6g m, %1.6g%%\n',re0,re0/norm(rf)*100);
fprintf('V_error = %1.6g m/sec, %1.6g%%\n',ve0,ve0/norm(vf)*100);

N = size(z0,1);

[zs,valid] = amsopt_cgls(fobj,z0,ppars,N,solpars);


opts = amsorb_top2_spool_z(zs,t0,tf);
outs = amsorb_top2_traj(opts,ppars);

%Initial solution statistics
fprintf('Initial solution stats:\n');
fprintf('dV = %1.6g m/sec J2 = %1.6g m^2/sec^3\n', out0.dV, out0.J2);
re0 = norm(out0.rlst{end}-rf);
ve0 = norm(out0.vlst{end}-vf);
fprintf('R_error = %1.6g m, %1.6g%%\n',re0,re0/norm(rf)*100);
fprintf('V_error = %1.6g m/sec, %1.6g%%\n',ve0,ve0/norm(vf)*100);
%Optimized solution statistics
fprintf('Optimized solution stats:\n');
fprintf('dV = %1.6g m/sec J2 = %1.6g m^2/sec^3\n', outs.dV, outs.J2);
res = norm(outs.rlst{end}-rf);
ves = norm(outs.vlst{end}-vf);
fprintf('R_error = %1.6g m, %1.6g%%\n',res,res/norm(rf)*100);
fprintf('V_error = %1.6g m/sec, %1.6g%%\n',ves,ves/norm(vf)*100);


%VII. Plot trajectory
for I = 1:size(outs.rlst,2)
    r2 = outs.rlst{I};
    xx2(I) = r2(1);
    yy2(I) = r2(2);
    zz2(I) = r2(3);
    
    r = out0.rlst{I};
    xx(I) = r(1);
    yy(I) = r(2);
    zz(I) = r(3);
end

for I = 1:size(outs.aulst,2)
    aun(I) = norm(outs.aulst{I});
end

figure(1);
clf; hold on;
    ams_draworbit(SS0.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SSf.objs{4}.eprop.oe,mu_sol,[0,0,1]);
    ams_draworbit(SS0.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    ams_draworbit(SSf.objs{5}.eprop.oe,mu_sol,[1,0,0]);
    plot3(0,0,0,'ko');
    
    %h = plot3(xx,yy,zz);
    %set(h,'Color',[0,1,0]);
    h = plot3(xx2,yy2,zz2);
    set(h,'Color',[0.5,0,0.5]);
    for I = 1:size(outs.aulst,2)
        if(mod(I,5)==0)
            ams_drawarrow(outs.rlst{I},outs.aulst{I}.*1E13,[0.5,0,0.5]);
        end
    end
    
    axis equal;
        
